<?php

namespace Drupal\feeds_filemaker\Feeds\Fetcher;

use Drupal\feeds\Exception\EmptyFeedException;
use Drupal\feeds\Feeds\Fetcher\HttpFetcher;
use Drupal\feeds\Result\HttpFetcherResult;
use Drupal\feeds\StateInterface;
use Drupal\feeds\FeedInterface;
use GuzzleHttp\Exception\RequestException;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\key\KeyRepositoryInterface;

/**
 * @FeedsFetcher(
 *   id = "filemaker_api_fetcher",
 *   title = @Translation("FileMaker API Fetcher"),
 *   description = @Translation("Fetches data from FileMaker API with authentication."),
 *   form = {
 *     "configuration" = "Drupal\feeds\Feeds\Fetcher\Form\HttpFetcherForm",
 *     "feed" = "Drupal\feeds\Feeds\Fetcher\Form\HttpFetcherFeedForm",
 *   }
 * )
 */
class FileMakerApiFetcher extends HttpFetcher implements ContainerFactoryPluginInterface {

  protected KeyRepositoryInterface $keyRepository;

  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->setKeyRepository($container->get('key.repository'));
    return $instance;
  }

  public function setKeyRepository(KeyRepositoryInterface $key_repository): self {
    $this->keyRepository = $key_repository;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function fetch(FeedInterface $feed, StateInterface $state) {
    $sink = $this->feedsFileSystem->tempnam($feed, 'filemaker_fetcher_');

    try {
      $token = $this->authenticate();
      $source = $feed->getSource();

      $options = [
        'headers' => [
          'Authorization' => 'Bearer ' . $token,
        ],
        'query' => [
          // @todo Change this
          '_limit' => 100,  // Adjust as needed
        ],
        'sink' => $sink,
      ];

      $response = $this->client->get($source, $options);

      // Check if the fetched data has any content
      $lengths = $response->getHeader('Content-Length');
      if (count($lengths) > 0) {
        $length = reset($lengths);
        if ($length < 1) {
          $this->fileSystem->unlink($sink);
          $state->setMessage($this->t('The feed is empty.'));
          throw new EmptyFeedException();
        }
      }

      return new HttpFetcherResult($sink, $response->getHeaders(), $this->fileSystem);
    }
    catch (RequestException $e) {
      $this->fileSystem->unlink($sink);
      $state->setMessage($this->t('The FileMaker API request failed with error: @error', [
        '@error' => $e->getMessage(),
      ]));
      throw new EmptyFeedException();
    }
  }

  protected function authenticate() {
    $auth_endpoint = $this->getCredential('filemaker_auth_endpoint');
    $username = $this->getCredential('filemaker_username');
    $password = $this->getCredential('filemaker_password');

    $response = $this->client->post($auth_endpoint, [
      'headers' => [
        'Authorization' => 'Basic ' . base64_encode("$username:$password"),
      ],
      'json' => [
        'fmDataSource' => [],
      ],
    ]);

    $body = json_decode($response->getBody(), TRUE);
    return $body['response']['token'];
  }

  protected function getCredential(string $key_id): string {
    $key = $this->keyRepository->getKey($key_id);
    if (!$key) {
      throw new \RuntimeException("Key '$key_id' not found.");
    }
    return $key->getKeyValue();
  }
}
